@@ -67,6 +67,7 @@
 
 #include "igt_core.h"
 #include "igt_aux.h"
+#include "igt_sysfs.h"
 
 #ifdef HAVE_LIBGEN_H
 #include <libgen.h>   /* for basename() on Solaris */
@@ -398,9 +399,6 @@ error:
 	return -errno;
 }
 
-#define MSEC_PER_SEC (1000)
-#define USEC_PER_SEC (1000*MSEC_PER_SEC)
-#define NSEC_PER_SEC (1000*USEC_PER_SEC)
 uint64_t igt_nsec_elapsed(struct timespec *start)
 {
 	struct timespec now;
@@ -515,6 +513,7 @@ static void common_exit_handler(int sig)
 {
 	if (!igt_only_list_subtests()) {
 		low_mem_killer_disable(false);
+		kick_fbcon(true);
 	}
 
 	/* When not killed by a signal check that igt_exit() has been properly
@@ -734,7 +733,8 @@ out:
 		exit(ret == -1 ? 0 : IGT_EXIT_INVALID);
 
 	if (!list_subtests) {
-		kmsg(KERN_INFO "%s: executing\n", command_str);
+		kick_fbcon(false);
+		kmsg(KERN_INFO "[IGT] %s: executing\n", command_str);
 		print_version();
 
 		oom_adjust_for_doom();
@@ -860,7 +860,7 @@ bool __igt_run_subtest(const char *subtest_name)
 		return false;
 	}
 
-	kmsg(KERN_INFO "%s: starting subtest %s\n", command_str, subtest_name);
+	kmsg(KERN_INFO "[IGT] %s: starting subtest %s\n", command_str, subtest_name);
 	igt_debug("Starting subtest: %s\n", subtest_name);
 
 	_igt_log_buffer_reset();
@@ -1122,7 +1122,7 @@ static void
 printnum(unsigned long long num, unsigned base)
 {
 	int i = 0;
-	unsigned long long __num;
+	unsigned long long __num = num;
 
 	/* determine from where we should start dividing */
 	do {
@@ -1157,14 +1157,14 @@ xprintfmt(const char *fmt, va_list ap)
 	unsigned long long num;
 
 	while (1) {
-		while ((ch = *(unsigned char *) fmt++) != '%') {
+		while ((ch = *(const unsigned char *) fmt++) != '%') {
 			if (ch == '\0') {
 				return;
 			}
 			xputch(ch);
 		}
 
-		ch = *(unsigned char *) fmt++;
+		ch = *(const unsigned char *) fmt++;
 		switch (ch) {
 		/* character */
 		case 'c':
@@ -1264,7 +1264,7 @@ static void print_backtrace_sig_safe(void)
 		unw_word_t off;
 
 		if (unw_get_proc_name(&cursor, name, 255, &off) < 0)
-			xstrlcpy(name, "<unknown>", 9);
+			xstrlcpy(name, "<unknown>", 10);
 
 		xprintf(" #%d [%s+0x%x]\n", stack_num++, name,
 				(unsigned int) off);
@@ -1343,7 +1343,7 @@ void igt_exit(void)
 	}
 
 	if (command_str)
-		kmsg(KERN_INFO "%s: exiting, ret=%d\n",
+		kmsg(KERN_INFO "[IGT] %s: exiting, ret=%d\n",
 		     command_str, igt_exitcode);
 	igt_debug("Exiting with status code %d\n", igt_exitcode);
 
@@ -1557,6 +1557,32 @@ bool __igt_fork(void)
 
 }
 
+/**
+ * igt_child_done:
+ *
+ * Lets the IGT core know that one of the children has exited.
+ */
+void igt_child_done(pid_t pid)
+{
+	int i = 0;
+	int found = -1;
+
+	igt_assert(num_test_children > 1);
+
+	for (i = 0; i < num_test_children; i++) {
+		if (pid == test_children[i]) {
+			found = i;
+			break;
+		}
+	}
+
+	igt_assert(found >= 0);
+
+	num_test_children--;
+	for (i = found; i < num_test_children; i++)
+		test_children[i] = test_children[i + 1];
+}
+
 /**
  * igt_waitchildren:
  *
@@ -1740,7 +1766,7 @@ static void fatal_sig_handler(int sig)
 			igt_assert_eq(write(STDERR_FILENO, ".\n", 2), 2);
 		}
 
-		if (in_subtest && crash_signal(sig)) {
+		if (crash_signal(sig)) {
 			/* Linux standard to return exit code as 128 + signal */
 			if (!failed_one)
 				igt_exitcode = 128 + sig;
@@ -1749,7 +1775,8 @@ static void fatal_sig_handler(int sig)
 #ifdef HAVE_LIBUNWIND
 			print_backtrace_sig_safe();
 #endif
-			exit_subtest("CRASH");
+			if (in_subtest)
+				exit_subtest("CRASH");
 		}
 		break;
 	}
