@@ -63,6 +63,8 @@ static int readN(int fd, char *buf, int len)
 			return total ?: ret;
 
 		total += ret;
+		if (total == len)
+			return total;
 	} while (1);
 }
 
@@ -78,6 +80,8 @@ static int writeN(int fd, const char *buf, int len)
 			return total ?: ret;
 
 		total += ret;
+		if (total == len)
+			return total;
 	} while (1);
 }
 
@@ -215,7 +219,7 @@ int igt_sysfs_read(int dir, const char *attr, void *data, int len)
 {
 	int fd;
 
-	fd = openat(dir, attr, O_WRONLY);
+	fd = openat(dir, attr, O_RDONLY);
 	if (fd < 0)
 		return false;
 
