@@ -41,14 +41,15 @@
 #include "media_fill.h"
 #include "ioctl_wrappers.h"
 #include "media_spin.h"
+#include "gpgpu_fill.h"
 
 #include <i915_drm.h>
 
 /**
  * SECTION:intel_batchbuffer
  * @short_description: Batchbuffer and blitter support
- * @title: intel batchbuffer
- * @include: intel_batchbuffer.h
+ * @title: Batch Buffer
+ * @include: igt.h
  *
  * This library provides some basic support for batchbuffers and using the
  * blitter engine based upon libdrm. A new batchbuffer is allocated with
@@ -439,7 +440,7 @@ intel_copy_bo(struct intel_batchbuffer *batch,
  * igt_buf_width:
  * @buf: the i-g-t buffer object
  *
- * Computes the widht in 32-bit pixels of the given buffer.
+ * Computes the width in 32-bit pixels of the given buffer.
  *
  * Returns:
  * The width of the buffer.
@@ -552,7 +553,7 @@ fill_object(struct drm_i915_gem_exec_object2 *obj, uint32_t gem_handle,
 	memset(obj, 0, sizeof(*obj));
 	obj->handle = gem_handle;
 	obj->relocation_count = count;
-	obj->relocs_ptr = (uintptr_t)relocs;
+	obj->relocs_ptr = to_user_pointer(relocs);
 }
 
 static void exec_blit(int fd,
@@ -561,7 +562,7 @@ static void exec_blit(int fd,
 {
 	struct drm_i915_gem_execbuffer2 exec;
 
-	exec.buffers_ptr = (uintptr_t)objs;
+	exec.buffers_ptr = to_user_pointer(objs);
 	exec.buffer_count = count;
 	exec.batch_start_offset = 0;
 	exec.batch_len = batch_len * 4;
@@ -674,7 +675,7 @@ void igt_blitter_fast_copy__raw(int fd,
  * @dst_x: destination pixel x-coordination
  * @dst_y: destination pixel y-coordination
  *
- * Copy @src into @dst using the gen9 fast copy blitter comamnd.
+ * Copy @src into @dst using the gen9 fast copy blitter command.
  *
  * The source and destination surfaces cannot overlap.
  */
@@ -786,6 +787,10 @@ igt_fillfunc_t igt_get_gpgpu_fillfunc(int devid)
 
 	if (IS_GEN7(devid))
 		fill = gen7_gpgpu_fillfunc;
+	else if (IS_BROADWELL(devid))
+		fill = gen8_gpgpu_fillfunc;
+	else if (IS_GEN9(devid))
+		fill = gen9_gpgpu_fillfunc;
 
 	return fill;
 }
