@@ -30,6 +30,7 @@
  * Testcase: run a couple of big batches to force the unbind on misalignment code.
  */
 
+#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -41,15 +42,19 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include "drm.h"
-#include "i915_drm.h"
-#include "drmtest.h"
+
+#include <drm.h>
+
+
+IGT_TEST_DESCRIPTION("Run a couple of big batches to force the unbind on"
+		     " misalignment code.");
 
 #define HEIGHT 256
 #define WIDTH 1024
 
 static void
-copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignment, int error)
+copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo,
+     uint64_t n_bo, uint64_t alignment, int error)
 {
 	uint32_t batch[12];
 	struct drm_i915_gem_relocation_entry reloc[2];
@@ -103,9 +108,9 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignme
 	}
 	obj[n].handle = handle;
 	obj[n].relocation_count = 2;
-	obj[n].relocs_ptr = (uintptr_t)reloc;
+	obj[n].relocs_ptr = to_user_pointer(reloc);
 
-	exec.buffers_ptr = (uintptr_t)obj;
+	exec.buffers_ptr = to_user_pointer(obj);
 	exec.buffer_count = n_bo + 1;
 	exec.batch_start_offset = 0;
 	exec.batch_len = i * 4;
@@ -119,18 +124,19 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo, int alignme
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	if (ret)
 		ret = errno;
-	igt_assert(ret == error);
+	igt_assert_eq(ret, error);
 
 	gem_close(fd, handle);
 	free(obj);
 }
 
-static void minor_evictions(int fd, int size, int count)
+static void minor_evictions(int fd, uint64_t size, uint64_t count)
 {
 	uint32_t *bo, *sel;
-	int n, m, alignment, pass, fail;
+	uint64_t n, m, alignment;
+	int pass, fail;
 
-	igt_require((uint64_t)count * size / (1024 * 1024) < intel_get_total_ram_mb() * 9 / 10);
+	intel_require_memory(2 * count, size, CHECK_RAM);
 
 	bo = malloc(3*count*sizeof(*bo));
 	igt_assert(bo);
@@ -155,12 +161,13 @@ static void minor_evictions(int fd, int size, int count)
 	free(bo);
 }
 
-static void major_evictions(int fd, int size, int count)
+static void major_evictions(int fd, uint64_t size, uint64_t count)
 {
-	int n, m, loop, alignment, max;
+	uint64_t n, m, alignment, max;
+	int loop;
 	uint32_t *bo;
 
-	igt_require((uint64_t)count * size / (1024 * 1024) < intel_get_total_ram_mb() * 9 / 10);
+	intel_require_memory(count, size, CHECK_RAM);
 
 	bo = malloc(count*sizeof(*bo));
 	igt_assert(bo);
@@ -181,19 +188,14 @@ static void major_evictions(int fd, int size, int count)
 	free(bo);
 }
 
-int fd;
-
 igt_main
 {
-	int size, count;
-
-	igt_skip_on_simulation();
+	uint64_t size, count;
+	int fd = -1;
 
 	igt_fixture {
-		fd = drm_open_any();
-
-		igt_skip_on_f(intel_gen(intel_get_drm_devid(fd)) > 7,
-			      "BLIT commands not yet updated\n");
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 	}
 
 	igt_subtest("minor-normal") {
@@ -220,8 +222,23 @@ igt_main
 		count = 4;
 		major_evictions(fd, size, count);
 	}
+
+	igt_subtest("minor-hang") {
+		igt_fork_hang_helper();
+		size = 1024 * 1024;
+		count = 3*gem_aperture_size(fd) / size / 4;
+		minor_evictions(fd, size, count);
+	}
+
+	igt_subtest("major-hang") {
+		size = 3*gem_aperture_size(fd) / 4;
+		count = 4;
+		major_evictions(fd, size, count);
+	}
 	igt_stop_signal_helper();
 
-	igt_fixture
+	igt_fixture {
+		igt_stop_hang_helper();
 		close(fd);
+	}
 }
