@@ -26,7 +26,6 @@
  */
 
 #define _GNU_SOURCE
-#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -39,10 +38,15 @@
 #include <sys/ioctl.h>
 #include "drm.h"
 
+#include "igt.h"
+#include "igt_x86.h"
+
 #ifndef PAGE_SIZE
 #define PAGE_SIZE 4096
 #endif
 
+#define abs(x) ((x) >= 0 ? (x) : -(x))
+
 static int OBJECT_SIZE = 16*1024*1024;
 
 static void
@@ -230,6 +234,59 @@ test_write(int fd)
 	munmap(src, OBJECT_SIZE);
 }
 
+static void
+test_wc(int fd)
+{
+	unsigned long gtt_reads, gtt_writes, cpu_writes;
+	uint32_t handle;
+	void *gtt, *cpu;
+
+	handle = gem_create(fd, 4096);
+	cpu = gem_mmap__cpu(fd, handle, 0, 4096, PROT_READ | PROT_WRITE);
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+	gem_close(fd, handle);
+
+	handle = gem_create(fd, 4096);
+	gtt = gem_mmap__gtt(fd, handle, 4096, PROT_READ | PROT_WRITE);
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	gem_close(fd, handle);
+
+	gtt_reads = 0;
+	igt_for_milliseconds(200) {
+		memcpy(cpu, gtt, 4096);
+		gtt_reads++;
+	}
+	igt_debug("%lu GTT reads in 200us\n", gtt_reads);
+
+	gtt_writes = 0;
+	igt_for_milliseconds(200) {
+		memcpy(gtt, cpu, 4096);
+		gtt_writes++;
+	}
+	igt_debug("%lu GTT writes in 200us\n", gtt_writes);
+
+	if (igt_setup_clflush()) {
+		cpu_writes = 0;
+		igt_for_milliseconds(200) {
+			igt_clflush_range(cpu, 4096);
+			cpu_writes++;
+		}
+		igt_debug("%lu CPU writes in 200us\n", cpu_writes);
+	} else
+		cpu_writes = gtt_writes;
+
+	munmap(cpu, 4096);
+	munmap(gtt, 4096);
+
+	igt_assert_f(gtt_writes > 2*gtt_reads,
+		     "Write-Combined writes are expected to be much faster than reads: read=%.2fMiB/s, write=%.2fMiB/s\n",
+		     5*gtt_reads/256., 5*gtt_writes/256.);
+
+	igt_assert_f(gtt_writes > cpu_writes/2,
+		     "Write-Combined writes are expected to be roughly equivalent to WB writes: WC (gtt)=%.2fMiB/s, WB (cpu)=%.2fMiB/s\n",
+		     5*gtt_writes/256., 5*cpu_writes/256.);
+}
+
 static void
 test_write_gtt(int fd)
 {
@@ -253,6 +310,142 @@ test_write_gtt(int fd)
 	munmap(src, OBJECT_SIZE);
 }
 
+static void
+test_coherency(int fd)
+{
+	uint32_t handle;
+	uint32_t *gtt, *cpu;
+	int i;
+
+	igt_require(igt_setup_clflush());
+
+	handle = gem_create(fd, OBJECT_SIZE);
+
+	gtt = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
+	cpu = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_READ | PROT_WRITE);
+	set_domain_gtt(fd, handle);
+
+	/* On byt/bsw/bxt this detects an interesting behaviour where the
+	 * CPU cannot flush the iobar and so the read may bypass the write.
+	 * https://bugs.freedesktop.org/show_bug.cgi?id=94314
+	 */
+	for (i = 0; i < OBJECT_SIZE / 64; i++) {
+		int x = 16*i + (i%16);
+		gtt[x] = i;
+		igt_clflush_range(&cpu[x], sizeof(cpu[x]));
+		igt_assert_eq(cpu[x], i);
+	}
+
+	munmap(cpu, OBJECT_SIZE);
+	munmap(gtt, OBJECT_SIZE);
+	gem_close(fd, handle);
+}
+
+static void
+test_clflush(int fd)
+{
+	uint32_t handle;
+	uint32_t *gtt;
+
+	igt_require(igt_setup_clflush());
+
+	handle = gem_create(fd, OBJECT_SIZE);
+
+	gtt = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_READ | PROT_WRITE);
+	set_domain_gtt(fd, handle);
+
+	igt_clflush_range(gtt, OBJECT_SIZE);
+
+	munmap(gtt, OBJECT_SIZE);
+	gem_close(fd, handle);
+}
+
+static void
+test_hang(int fd)
+{
+	igt_hang_t hang;
+	uint32_t patterns[] = {
+		0, 0xaaaaaaaa, 0x55555555, 0xcccccccc,
+	};
+	uint32_t *gtt[3];
+	int last_pattern = 0;
+	int next_pattern = 1;
+	int i;
+
+	for (i = I915_TILING_NONE; i <= I915_TILING_Y; i++) {
+		uint32_t handle;
+
+		handle = gem_create(fd, OBJECT_SIZE);
+		gem_set_tiling(fd, handle, i, 2048);
+
+		gtt[i] = gem_mmap__gtt(fd, handle, OBJECT_SIZE, PROT_WRITE);
+		set_domain_gtt(fd, handle);
+		gem_close(fd, handle);
+	}
+
+	hang = igt_hang_ring(fd, I915_EXEC_RENDER);
+
+	do {
+		for (i = 0; i < OBJECT_SIZE / 64; i++) {
+			int x = 16*i + (i%16);
+
+			igt_assert(gtt[0][x] == patterns[last_pattern]);
+			igt_assert(gtt[1][x] == patterns[last_pattern]);
+			igt_assert(gtt[2][x] == patterns[last_pattern]);
+
+			gtt[0][x] = patterns[next_pattern];
+			gtt[1][x] = patterns[next_pattern];
+			gtt[2][x] = patterns[next_pattern];
+		}
+
+		last_pattern = next_pattern;
+		next_pattern = (next_pattern + 1) % ARRAY_SIZE(patterns);
+	} while (gem_bo_busy(fd, hang.handle));
+
+	igt_post_hang_ring(fd, hang);
+
+	munmap(gtt[0], OBJECT_SIZE);
+	munmap(gtt[1], OBJECT_SIZE);
+	munmap(gtt[2], OBJECT_SIZE);
+}
+
+static int min_tile_width(uint32_t devid, int tiling)
+{
+	if (tiling < 0) {
+		if (intel_gen(devid) >= 4)
+			return 4096 - min_tile_width(devid, -tiling);
+		else
+			return 1024;
+
+	}
+
+	if (intel_gen(devid) == 2)
+		return 128;
+	else if (tiling == I915_TILING_X)
+		return 512;
+	else if (IS_915(devid))
+		return 512;
+	else
+		return 128;
+}
+
+static int max_tile_width(uint32_t devid, int tiling)
+{
+	if (tiling < 0) {
+		if (intel_gen(devid) >= 4)
+			return 4096 + min_tile_width(devid, -tiling);
+		else
+			return 2048;
+	}
+
+	if (intel_gen(devid) >= 7)
+		return 256 << 10;
+	else if (intel_gen(devid) >= 4)
+		return 128 << 10;
+	else
+		return 8 << 10;
+}
+
 static void
 test_huge_bo(int fd, int huge, int tiling)
 {
@@ -261,18 +454,31 @@ test_huge_bo(int fd, int huge, int tiling)
 	char *tiled_pattern;
 	char *linear_pattern;
 	uint64_t size, last_offset;
-	int pitch = tiling == I915_TILING_Y ? 128 : 512;
+	uint32_t devid = intel_get_drm_devid(fd);
+	int pitch = min_tile_width(devid, tiling);
 	int i;
 
 	switch (huge) {
 	case -1:
 		size = gem_mappable_aperture_size() / 2;
+
+		/* Power of two fence size, natural fence
+		 * alignment, and the guard page at the end
+		 * gtt means that if the entire gtt is
+		 * mappable, we can't usually fit in a tiled
+		 * object half the size of the gtt. Let's use
+		 * a quarter size one instead.
+		 */
+		if (tiling &&
+		    intel_gen(intel_get_drm_devid(fd)) < 4 &&
+		    size >= gem_global_aperture_size(fd) / 2)
+			size /= 2;
 		break;
 	case 0:
 		size = gem_mappable_aperture_size() + PAGE_SIZE;
 		break;
 	default:
-		size = gem_aperture_size(fd) + PAGE_SIZE;
+		size = gem_global_aperture_size(fd) + PAGE_SIZE;
 		break;
 	}
 	intel_require_memory(1, size, CHECK_RAM);
@@ -324,12 +530,62 @@ test_huge_bo(int fd, int huge, int tiling)
 	munmap(linear_pattern, PAGE_SIZE);
 }
 
-static void
-test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
+#if defined(__x86_64__) && !defined(__clang__)
+#define MOVNT 512
+
+#pragma GCC push_options
+#pragma GCC target("sse4.1")
+
+#include <smmintrin.h>
+__attribute__((noinline))
+static void copy_wc_page(void *dst, void *src)
 {
+	if (igt_x86_features() & SSE4_1) {
+		__m128i *S = (__m128i *)src;
+		__m128i *D = (__m128i *)dst;
+
+		for (int i = 0; i < PAGE_SIZE/64; i++) {
+			__m128i tmp[4];
+
+			tmp[0] = _mm_stream_load_si128(S++);
+			tmp[1] = _mm_stream_load_si128(S++);
+			tmp[2] = _mm_stream_load_si128(S++);
+			tmp[3] = _mm_stream_load_si128(S++);
+
+			_mm_store_si128(D++, tmp[0]);
+			_mm_store_si128(D++, tmp[1]);
+			_mm_store_si128(D++, tmp[2]);
+			_mm_store_si128(D++, tmp[3]);
+		}
+	} else
+		memcpy(dst, src, PAGE_SIZE);
+}
+
+#pragma GCC pop_options
+
+#else
+static void copy_wc_page(void *dst, const void *src)
+{
+	memcpy(dst, src, PAGE_SIZE);
+}
+#endif
+
+static unsigned int tile_row_size(int tiling, unsigned int stride)
+{
+	if (tiling < 0)
+		tiling = -tiling;
+
+	return stride * (tiling == I915_TILING_Y ? 32 : 8);
+}
+
+#define rounddown(x, y) (x - (x%y))
+
+static void
+test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
+{
+	const uint32_t devid = intel_get_drm_devid(fd);
 	uint64_t huge_object_size, i;
-	uint32_t bo, *pattern_a, *pattern_b;
-	char *a, *b;
+	unsigned mode = CHECK_RAM;
 
 	switch (huge) {
 	case -2:
@@ -341,65 +597,98 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
 	case 0:
 		huge_object_size = gem_mappable_aperture_size() + PAGE_SIZE;
 		break;
+	case 1:
+		huge_object_size = gem_global_aperture_size(fd) + PAGE_SIZE;
+		break;
 	default:
-		huge_object_size = gem_aperture_size(fd) + PAGE_SIZE;
+		huge_object_size = (intel_get_total_ram_mb() << 19) + PAGE_SIZE;
+		mode |= CHECK_SWAP;
 		break;
 	}
-	intel_require_memory(2, huge_object_size, CHECK_RAM);
+	intel_require_memory(2*ncpus, huge_object_size, mode);
 
-	pattern_a = malloc(PAGE_SIZE);
-	for (i = 0; i < PAGE_SIZE/4; i++)
-		pattern_a[i] = i;
+	igt_fork(child, ncpus) {
+		uint64_t valid_size = huge_object_size;
+		uint32_t bo;
+		char *a, *b;
 
-	pattern_b = malloc(PAGE_SIZE);
-	for (i = 0; i < PAGE_SIZE/4; i++)
-		pattern_b[i] = ~i;
+		bo = gem_create(fd, huge_object_size);
+		if (tiling_a) {
+			igt_require(__gem_set_tiling(fd, bo, abs(tiling_a), min_tile_width(devid, tiling_a)) == 0);
+			valid_size = rounddown(valid_size, tile_row_size(tiling_a, min_tile_width(devid, tiling_a)));
+		}
+		a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+		igt_require(a);
+		gem_close(fd, bo);
 
-	bo = gem_create(fd, huge_object_size);
-	if (tiling_a)
-		igt_require(__gem_set_tiling(fd, bo, tiling_a, tiling_a == I915_TILING_Y ? 128 : 512) == 0);
-	a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
-	igt_require(a);
-	gem_close(fd, bo);
+		bo = gem_create(fd, huge_object_size);
+		if (tiling_b) {
+			igt_require(__gem_set_tiling(fd, bo, abs(tiling_b), max_tile_width(devid, tiling_b)) == 0);
+			valid_size = rounddown(valid_size, tile_row_size(tiling_b, max_tile_width(devid, tiling_b)));
+		}
+		b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+		igt_require(b);
+		gem_close(fd, bo);
 
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
-		memcpy(a + PAGE_SIZE*i, pattern_a, PAGE_SIZE);
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t *ptr = (uint32_t *)(a + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				ptr[j] = i + j;
+			igt_progress("Writing a ", i, valid_size / PAGE_SIZE);
+		}
 
-	bo = gem_create(fd, huge_object_size);
-	if (tiling_b)
-		igt_require(__gem_set_tiling(fd, bo, tiling_b, tiling_b == I915_TILING_Y ? 128 : 512) == 0);
-	b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
-	igt_require(b);
-	gem_close(fd, bo);
 
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
-		memcpy(b + PAGE_SIZE*i, pattern_b, PAGE_SIZE);
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t *ptr = (uint32_t *)(b + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				ptr[j] = ~(i + j);
+			igt_progress("Writing b ", i, valid_size / PAGE_SIZE);
+		}
 
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			memcpy(a + i *PAGE_SIZE, b + i*PAGE_SIZE, PAGE_SIZE);
-		else
-			memcpy(b + i *PAGE_SIZE, a + i*PAGE_SIZE, PAGE_SIZE);
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t *A = (uint32_t *)(a + PAGE_SIZE*i);
+			uint32_t *B = (uint32_t *)(b + PAGE_SIZE*i);
+			uint32_t A_tmp[PAGE_SIZE/sizeof(uint32_t)];
+			uint32_t B_tmp[PAGE_SIZE/sizeof(uint32_t)];
+
+			copy_wc_page(A_tmp, A);
+			copy_wc_page(B_tmp, B);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				if ((i +  j) & 1)
+					A_tmp[j] = B_tmp[j];
+				else
+					B_tmp[j] = A_tmp[j];
+			memcpy(A, A_tmp, PAGE_SIZE);
+			memcpy(B, B_tmp, PAGE_SIZE);
+
+			igt_progress("Copying a<->b ", i, valid_size / PAGE_SIZE);
+		}
+
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
+			copy_wc_page(page, a + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
+				if ((i + j) & 1)
+					igt_assert_eq_u32(page[j], ~(i + j));
+				else
+					igt_assert_eq_u32(page[j], i + j);
+			igt_progress("Checking a ", i, valid_size / PAGE_SIZE);
+		}
+		munmap(a, huge_object_size);
+
+		for (i = 0; i < valid_size / PAGE_SIZE; i++) {
+			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
+			copy_wc_page(page, b + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
+				if ((i + j) & 1)
+					igt_assert_eq_u32(page[j], ~(i + j));
+				else
+					igt_assert_eq_u32(page[j], i + j);
+			igt_progress("Checking b ", i, valid_size / PAGE_SIZE);
+		}
+		munmap(b, huge_object_size);
 	}
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			igt_assert(memcmp(pattern_b, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		else
-			igt_assert(memcmp(pattern_a, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
-	}
-	munmap(a, huge_object_size);
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			igt_assert(memcmp(pattern_b, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		else
-			igt_assert(memcmp(pattern_a, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
-	}
-	munmap(b, huge_object_size);
-
-	free(pattern_a);
-	free(pattern_b);
+	igt_waitchildren();
 }
 
 static void
@@ -520,6 +809,12 @@ igt_main
 		test_write(fd);
 	igt_subtest("basic-write-gtt")
 		test_write_gtt(fd);
+	igt_subtest("coherency")
+		test_coherency(fd);
+	igt_subtest("clflush")
+		test_clflush(fd);
+	igt_subtest("hang")
+		test_hang(fd);
 	igt_subtest("basic-read-write")
 		test_read_write(fd, READ_BEFORE_WRITE);
 	igt_subtest("basic-write-read")
@@ -538,6 +833,8 @@ igt_main
 		run_without_prefault(fd, test_write_gtt);
 	igt_subtest("basic-write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
+	igt_subtest("basic-wc")
+		test_wc(fd);
 
 	igt_subtest("basic-small-bo")
 		test_huge_bo(fd, -1, I915_TILING_NONE);
@@ -560,22 +857,47 @@ igt_main
 	igt_subtest("huge-bo-tiledY")
 		test_huge_bo(fd, 1, I915_TILING_Y);
 
-	igt_subtest("basic-small-copy")
-		test_huge_copy(fd, -2, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("basic-small-copy-XY")
-		test_huge_copy(fd, -2, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("medium-copy")
-		test_huge_copy(fd, -1, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("medium-copy-XY")
-		test_huge_copy(fd, -1, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("big-copy")
-		test_huge_copy(fd, 0, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("big-copy-XY")
-		test_huge_copy(fd, 0, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("huge-copy")
-		test_huge_copy(fd, 1, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("huge-copy-XY")
-		test_huge_copy(fd, 1, I915_TILING_X, I915_TILING_Y);
+	igt_subtest_group {
+		const struct copy_size {
+			const char *prefix;
+			int size;
+		} copy_sizes[] = {
+			{ "basic-small", -2 },
+			{ "medium", -1 },
+			{ "big", 0 },
+			{ "huge", 1 },
+			{ "swap", 2 },
+			{ }
+		};
+		const struct copy_mode {
+			const char *suffix;
+			int tiling_x, tiling_y;
+		} copy_modes[] = {
+			{ "", I915_TILING_NONE, I915_TILING_NONE},
+			{ "-XY", I915_TILING_X, I915_TILING_Y},
+			{ "-odd", -I915_TILING_X, -I915_TILING_Y},
+			{}
+		};
+		const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+
+		for (const struct copy_size *s = copy_sizes; s->prefix; s++)
+			for (const struct copy_mode *m = copy_modes; m->suffix; m++) {
+				igt_subtest_f("%s-copy%s", s->prefix, m->suffix)
+					test_huge_copy(fd,
+							s->size,
+							m->tiling_x,
+							m->tiling_y,
+							1);
+
+				igt_subtest_f("forked-%s-copy%s", s->prefix, m->suffix)
+					test_huge_copy(fd,
+							s->size,
+							m->tiling_x,
+							m->tiling_y,
+							ncpus);
+			}
+	}
+
 
 	igt_fixture
 		close(fd);
