@@ -169,11 +169,11 @@ static void reloc_and_emit(int fd, drm_intel_bo *target_bo, bool faulting_reloc)
 	exec[1].relocation_count = 1;
 	/* A newly mmap gtt bo will fault on first access. */
 	if (faulting_reloc)
-		exec[1].relocs_ptr = (uintptr_t)gtt_relocs;
+		exec[1].relocs_ptr = to_user_pointer(gtt_relocs);
 	else
-		exec[1].relocs_ptr = (uintptr_t)reloc;
+		exec[1].relocs_ptr = to_user_pointer(reloc);
 
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_len = special_batch_len;
 	if (intel_gen(devid) >= 6)
@@ -184,18 +184,18 @@ static void reloc_and_emit(int fd, drm_intel_bo *target_bo, bool faulting_reloc)
 	gem_close(fd, handle_relocs);
 }
 
-static struct igt_hang_ring no_hang(int fd)
+static igt_hang_t no_hang(int fd)
 {
-	return (struct igt_hang_ring){0};
+	return (igt_hang_t){0};
 }
 
-static struct igt_hang_ring bcs_hang(int fd)
+static igt_hang_t bcs_hang(int fd)
 {
 	return igt_hang_ring(fd, I915_EXEC_BLT);
 }
 
 static void do_test(int fd, bool faulting_reloc,
-		    struct igt_hang_ring (*do_hang)(int fd))
+		    igt_hang_t (*do_hang)(int fd))
 {
 	uint32_t tiling_mode = I915_TILING_X;
 	unsigned long pitch, act_size;
@@ -214,7 +214,7 @@ static void do_test(int fd, bool faulting_reloc,
 	create_special_bo();
 
 	for (i = 0; i < NUM_TARGET_BOS; i++) {
-		struct igt_hang_ring hang;
+		igt_hang_t hang;
 
 		pc_target_bo[i] = drm_intel_bo_alloc(bufmgr, "special batch", 4096, 4096);
 		emit_dummy_load(pitch);
@@ -265,7 +265,7 @@ static void do_forked_test(int fd, unsigned flags)
 		igt_fork_helper(&thrasher) {
 			while (1) {
 				usleep(1000);
-				igt_drop_caches_set(val);
+				igt_drop_caches_set(fd, val);
 			}
 		}
 	}
@@ -301,6 +301,7 @@ igt_main
 
 	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		/* disable reuse, otherwise the test fails */
 		//drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -318,18 +319,14 @@ igt_main
 	igt_subtest("interruptible")
 		do_test(fd, false, no_hang);
 
-	igt_subtest("interruptible-hang") {
-		igt_require_hang_ring(fd, I915_EXEC_BLT);
+	igt_subtest("interruptible-hang")
 		do_test(fd, false, bcs_hang);
-	}
 
 	igt_subtest("faulting-reloc-interruptible")
 		do_test(fd, true, no_hang);
 
-	igt_subtest("faulting-reloc-interruptible-hang") {
-		igt_require_hang_ring(fd, I915_EXEC_BLT);
+	igt_subtest("faulting-reloc-interruptible-hang")
 		do_test(fd, true, bcs_hang);
-	}
 	igt_stop_signal_helper();
 
 	for (unsigned flags = 0; flags <= ALL_FLAGS; flags++) {
