@@ -85,7 +85,7 @@ static int noop(int fd, uint32_t ctx, const struct intel_execution_engine *e)
 	gem_write(fd, exec.handle, 0, &bbe, sizeof(bbe));
 
 	memset(&eb, 0, sizeof(eb));
-	eb.buffers_ptr = (uintptr_t)&exec;
+	eb.buffers_ptr = to_user_pointer(&exec);
 	eb.buffer_count = 1;
 	eb.flags = e->exec_id | e->flags;
 	i915_execbuffer2_set_context_id(eb, ctx);
@@ -156,7 +156,7 @@ static void inject_hang(int fd, uint32_t ctx,
 			const struct intel_execution_engine *e,
 			unsigned flags)
 {
-	igt_hang_ring_t hang;
+	igt_hang_t hang;
 
 	clock_gettime(CLOCK_MONOTONIC, &ts_injected);
 
@@ -522,7 +522,7 @@ static void noop_on_each_ring(int fd, const bool reverse)
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
 
 	memset(&eb, 0, sizeof(eb));
-	eb.buffers_ptr = (uintptr_t)&obj;
+	eb.buffers_ptr = to_user_pointer(&obj);
 	eb.buffer_count = 1;
 
 	if (reverse) {
@@ -547,7 +547,7 @@ static void test_close_pending_fork(const struct intel_execution_engine *e,
 				    const bool reverse)
 {
 	int fd = drm_open_driver(DRIVER_INTEL);
-	igt_hang_ring_t hang;
+	igt_hang_t hang;
 	int pid;
 
 	assert_reset_status(fd, fd, 0, RS_NO_ERROR);
