@@ -166,7 +166,7 @@ static igt_output_t *set_fb_on_crtc(igt_display_t *display, int pipe, struct igt
 			      mode->hdisplay, mode->vdisplay,
 			      DRM_FORMAT_XRGB8888, I915_TILING_NONE, fb_info);
 
-		primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+		primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 		igt_plane_set_fb(primary, fb_info);
 
 		return output;
@@ -180,7 +180,7 @@ static void set_cursor_on_pipe(igt_display_t *display, enum pipe pipe, struct ig
 	igt_plane_t *plane, *cursor = NULL;
 
 	for_each_plane_on_pipe(display, pipe, plane) {
-		if (!plane->is_cursor)
+		if (plane->type != DRM_PLANE_TYPE_CURSOR)
 			continue;
 
 		cursor = plane;
@@ -250,13 +250,14 @@ static enum pipe find_connected_pipe(igt_display_t *display, bool second)
 	return pipe;
 }
 
-static void flip_nonblocking(igt_display_t *display, enum pipe pipe, bool atomic, struct igt_fb *fb)
+static void flip_nonblocking(igt_display_t *display, enum pipe pipe_id, bool atomic, struct igt_fb *fb)
 {
-	igt_plane_t *primary = &display->pipes[pipe].planes[IGT_PLANE_PRIMARY];
+	igt_pipe_t *pipe = &display->pipes[pipe_id];
+	igt_plane_t *primary = igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_PRIMARY);
 
 	if (!atomic) {
 		/* Schedule a nonblocking flip for the next vblank */
-		do_or_die(drmModePageFlip(display->drm_fd, display->pipes[pipe].crtc_id, fb->fb_id,
+		do_or_die(drmModePageFlip(display->drm_fd, pipe->crtc_id, fb->fb_id,
 					DRM_MODE_PAGE_FLIP_EVENT, fb));
 	} else {
 		igt_plane_set_fb(primary, fb);
@@ -274,21 +275,55 @@ enum flip_test {
 	flip_test_last = flip_test_atomic_transitions_varying_size
 };
 
-static void transition_nonblocking(igt_display_t *display, enum pipe pipe,
+static bool cursor_slowpath(enum flip_test mode)
+{
+	/* cursor moving doesn't take slowpath, everything else does. */
+	if (mode == flip_test_legacy || mode == flip_test_atomic)
+		return false;
+
+	return true;
+}
+
+/*
+ * On platforms with two-stage watermark programming
+ * changing sprite visibility may require a extra vblank wait.
+ *
+ * Handle this here.
+ */
+static bool mode_requires_extra_vblank(enum flip_test mode)
+{
+	if (mode == flip_test_atomic_transitions ||
+	    mode == flip_test_atomic_transitions_varying_size)
+		return true;
+
+	return false;
+}
+
+static void transition_nonblocking(igt_display_t *display, enum pipe pipe_id,
 				   struct igt_fb *prim_fb, struct igt_fb *argb_fb,
 				   bool hide_sprite)
 {
-	igt_plane_t *primary = &display->pipes[pipe].planes[IGT_PLANE_PRIMARY];
-	igt_plane_t *sprite = &display->pipes[pipe].planes[IGT_PLANE_2];
+	igt_pipe_t *pipe = &display->pipes[pipe_id];
+	igt_plane_t *primary = igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_PRIMARY);
+	igt_plane_t *sprite = igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_OVERLAY);
 
 	if (hide_sprite) {
 		igt_plane_set_fb(primary, prim_fb);
 		igt_plane_set_fb(sprite, NULL);
 	} else {
+		int ret;
+
 		igt_plane_set_fb(primary, NULL);
 		igt_plane_set_fb(sprite, argb_fb);
-	}
 
+		ret = igt_display_try_commit_atomic(display, DRM_MODE_ATOMIC_NONBLOCK | DRM_MODE_PAGE_FLIP_EVENT, display);
+		if (!ret)
+			return;
+
+		igt_assert(ret == -EINVAL);
+
+		igt_plane_set_fb(sprite, prim_fb);
+	}
 	igt_display_commit_atomic(display, DRM_MODE_ATOMIC_NONBLOCK | DRM_MODE_PAGE_FLIP_EVENT, display);
 }
 
@@ -336,7 +371,7 @@ static void prepare_flip_test(igt_display_t *display,
 	if (mode == flip_test_atomic_transitions ||
 	    mode == flip_test_atomic_transitions_varying_size) {
 		igt_require(display->pipes[flip_pipe].n_planes > 1 &&
-			    !display->pipes[flip_pipe].planes[IGT_PLANE_2].is_cursor);
+		            display->pipes[flip_pipe].planes[1].type != DRM_PLANE_TYPE_CURSOR);
 
 		igt_create_color_pattern_fb(display->drm_fd, prim_fb->width, prim_fb->height,
 					    DRM_FORMAT_ARGB8888, 0, .1, .1, .1, argb_fb);
@@ -555,6 +590,7 @@ static void basic_flip_cursor(igt_display_t *display,
 	unsigned vblank_start;
 	enum pipe pipe = find_connected_pipe(display, false);
 	uint32_t *busy;
+	int i, miss1 = 0, miss2 = 0, delta;
 
 	if (mode >= flip_test_atomic)
 		igt_require(display->is_atomic);
@@ -575,59 +611,89 @@ static void basic_flip_cursor(igt_display_t *display,
 	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
-	/* Bind the cursor first to warm up */
-	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-
-	/* Start with a synchronous query to align with the vblank */
-	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-
-	busy = NULL;
-	if (flags & BASIC_BUSY)
-		busy = make_fb_busy(display->drm_fd, &fb_info);
-
-	switch (order) {
-	case FLIP_BEFORE_CURSOR:
-		switch (mode) {
-		default:
-			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
-			break;
-		case flip_test_atomic_transitions:
-		case flip_test_atomic_transitions_varying_size:
-			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
-			break;
-		}
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	for (i = 0; i < 25; i++) {
+		bool miss;
 
+		/* Bind the cursor first to warm up */
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-		break;
 
-	case FLIP_AFTER_CURSOR:
-		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		busy = NULL;
+		if (flags & BASIC_BUSY)
+			busy = make_fb_busy(display->drm_fd, &fb_info);
 
-		switch (mode) {
-		default:
-			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
-			break;
-		case flip_test_atomic_transitions:
-		case flip_test_atomic_transitions_varying_size:
-			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+		/* Start with a synchronous query to align with the vblank */
+		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+		switch (order) {
+		case FLIP_BEFORE_CURSOR:
+			switch (mode) {
+			default:
+				flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+				break;
+			case flip_test_atomic_transitions:
+			case flip_test_atomic_transitions_varying_size:
+				transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+				break;
+			}
+
+			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			miss = delta != 0;
+
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 			break;
+
+		case FLIP_AFTER_CURSOR:
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+
+			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			miss = delta != 0;
+
+			switch (mode) {
+			default:
+				flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+				break;
+			case flip_test_atomic_transitions:
+			case flip_test_atomic_transitions_varying_size:
+				transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+				break;
+			}
 		}
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+
+		if (busy) {
+			struct pollfd pfd = { display->drm_fd, POLLIN };
+			igt_assert(poll(&pfd, 1, 0) == 0);
+			finish_fb_busy(busy);
+		}
+
+		if (miss)
+			{ /* compare nothing, already failed */ }
+		else if (!cursor_slowpath(mode))
+			miss = delta != 0;
+		else
+			miss = delta != 0 && delta != 1;
+
+		miss1 += miss;
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+
+		if (miss1)
+			continue;
+
+		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+
+		if (!mode_requires_extra_vblank(mode))
+			miss2 += delta != 1;
+		else
+			miss2 += delta != 1 && delta != 2;
 	}
 
-	if (busy) {
-		struct pollfd pfd = { display->drm_fd, POLLIN };
-		igt_assert(poll(&pfd, 1, 0) == 0);
-		finish_fb_busy(busy);
-	}
-
-	igt_set_timeout(1, "Stuck page flip");
-	igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
-	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
-	igt_reset_timeout();
+	igt_fail_on_f(miss1 > 2 || miss1 + miss2 > 5, "Failed to evade %i vblanks and missed %i page flips\n", miss1, miss2);
+	if (miss1 || miss2)
+		igt_info("Failed to evade %i vblanks and missed %i page flips\n", miss1, miss2);
 
 	do_cleanup_display(display);
 	igt_remove_fb(display->drm_fd, &fb_info);
@@ -639,14 +705,45 @@ static void basic_flip_cursor(igt_display_t *display,
 		igt_remove_fb(display->drm_fd, &cursor_fb2);
 }
 
+static int
+get_cursor_updates_per_vblank(igt_display_t *display, enum pipe pipe,
+			      struct drm_mode_cursor *arg)
+{
+	int target;
+
+	for (target = 65536; target; target /= 2) {
+		unsigned vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		for (int n = 0; n < target; n++)
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, arg);
+		if (get_vblank(display->drm_fd, pipe, 0) == vblank_start)
+			break;
+	}
+
+	/*
+	  * Divide by 4, to handle variations in amount of vblanks
+	  * caused by cpufreq throttling.
+	  */
+	target /= 4;
+	igt_require(target > 1);
+
+	igt_debug("Using a target of %d cursor updates per quarter-vblank\n", target);
+
+	return target;
+}
+
 static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloops)
 {
 	struct drm_mode_cursor arg[2];
 	struct drm_event_vblank vbl;
 	struct igt_fb fb_info, cursor_fb, cursor_fb2, argb_fb;
 	unsigned vblank_start;
-	int target;
+	int target, cpu;
 	enum pipe pipe = find_connected_pipe(display, false);
+	volatile unsigned long *shared;
+	cpu_set_t mask, oldmask;
 
 	if (mode >= flip_test_atomic)
 		igt_require(display->is_atomic);
@@ -661,21 +758,9 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	if (nloops) {
-		target = 4096;
-		do {
-			vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-			for (int n = 0; n < target; n++)
-				do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-			target /= 2;
-			if (get_vblank(display->drm_fd, pipe, 0) == vblank_start)
-				break;
-		} while (target);
-		igt_require(target > 1);
-
-		igt_debug("Using a target of %d cursor updates per half-vblank\n", target);
-	} else
+	if (nloops)
+		target = get_cursor_updates_per_vblank(display, pipe, &arg[0]);
+	else
 		target = 1;
 
 	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
@@ -684,6 +769,37 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
+	/*
+	 * There are variations caused by using cpu frequency changing. To
+	 * eliminate those we force this test to run on the same cpu as an
+	 * idle thread that does a busy loop of sched_yield(); The effect is
+	 * that we don't throttle the cpu to a lower frequency, and the
+	 * variations caused by cpu speed changing are eliminated.
+	 */
+	if (target > 1) {
+		shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+		igt_assert(shared != MAP_FAILED);
+
+		cpu = sched_getcpu();
+		igt_assert(cpu >= 0);
+
+		CPU_ZERO(&mask);
+		CPU_SET(cpu, &mask);
+		sched_getaffinity(0, sizeof(oldmask), &oldmask);
+		sched_setaffinity(0, sizeof(mask), &mask);
+
+		shared[0] = 0;
+
+		igt_fork(child, 1) {
+			struct sched_param parm = { .sched_priority = 0 };
+
+			igt_assert(sched_setscheduler(0, SCHED_IDLE, &parm) == 0);
+
+			while (!shared[0])
+				sched_yield();
+		}
+	}
+
 	do {
 		/* Bind the cursor first to warm up */
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
@@ -704,15 +820,31 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 		for (int n = 0; n < target; n++)
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
+
 		/* Nor should it have delayed the following cursor update */
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		if (!cursor_slowpath(mode))
+			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		else
+			igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
 
 		igt_set_timeout(1, "Stuck page flip");
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+
+		if (!mode_requires_extra_vblank(mode))
+			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+		else
+			igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 2);
+
 		igt_reset_timeout();
 	} while (nloops--);
 
+	if (target > 1) {
+		shared[0] = 1;
+		igt_waitchildren();
+		munmap((void *)shared, 4096);
+		sched_setaffinity(0, sizeof(oldmask), &oldmask);
+	}
+
 	do_cleanup_display(display);
 	igt_remove_fb(display->drm_fd, &fb_info);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
@@ -749,7 +881,7 @@ static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
 	struct igt_fb fb_info, cursor_fb;
 	igt_output_t *output;
 	enum pipe pipe = find_connected_pipe(display, false);
-	struct drm_mode_cursor arg;
+	struct drm_mode_cursor arg[2];
 	bool skip_test;
 	igt_plane_t *cursor = NULL, *plane;
 
@@ -757,11 +889,11 @@ static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
 	igt_require((output = set_fb_on_crtc(display, pipe, &fb_info)));
 	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
-	populate_cursor_args(display, pipe, &arg, &cursor_fb);
-	arg.flags |= DRM_MODE_CURSOR_BO;
+	populate_cursor_args(display, pipe, arg, &cursor_fb);
+	arg[0].flags |= DRM_MODE_CURSOR_BO;
 
 	for_each_plane_on_pipe(display, pipe, plane) {
-		if (!plane->is_cursor)
+		if (plane->type != DRM_PLANE_TYPE_CURSOR)
 			continue;
 
 		cursor = plane;
@@ -802,7 +934,7 @@ static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
 		igt_assert_eq(0, poll(&pfd, 1, 0));
 		igt_assert_eq(0, pfd.revents);
 
-		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 
 		igt_assert_eq(1, poll(&pfd, 1, 0));
 		igt_assert_eq(POLLIN, pfd.revents);
@@ -818,7 +950,7 @@ static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
 		igt_assert_eq(0, pfd.revents);
 
 		/* Same for cursor on disabled crtc. */
-		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 
 		igt_assert_eq(1, poll(&pfd, 1, 0));
 		igt_assert_eq(POLLIN, pfd.revents);
@@ -837,7 +969,7 @@ cleanup:
 		igt_skip("Nonblocking modeset is not supported by this kernel\n");
 }
 
-static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool modeset)
+static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool modeset, bool atomic)
 {
 	struct drm_mode_cursor arg[2], arg2[2];
 	struct drm_event_vblank vbl;
@@ -845,29 +977,27 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 	unsigned vblank_start;
 	enum pipe pipe = find_connected_pipe(display, false);
 	enum pipe pipe2 = find_connected_pipe(display, true);
-	igt_output_t *output2;
+	igt_output_t *output, *output2;
 	bool skip_test = false;
 
-	if (modeset)
+	igt_fail_on(modeset && !atomic);
+
+	if (atomic)
 		igt_require(display->is_atomic);
 
-	igt_require(set_fb_on_crtc(display, pipe, &fb_info));
+	igt_require((output = set_fb_on_crtc(display, pipe, &fb_info)));
 	igt_require((output2 = set_fb_on_crtc(display, pipe2, &fb2_info)));
 
 	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
 	populate_cursor_args(display, pipe, arg, &cursor_fb);
 
-	arg[0].flags = arg[1].flags = DRM_MODE_CURSOR_BO;
-	arg[1].handle = 0;
-	arg[1].width = arg[1].height = 0;
+	arg[1].x = arg[1].y = 192;
 
 	set_cursor_on_pipe(display, pipe2, &cursor_fb);
 	populate_cursor_args(display, pipe2, arg2, &cursor_fb);
 
-	arg2[0].flags = arg2[1].flags = DRM_MODE_CURSOR_BO;
-	arg2[0].handle = 0;
-	arg2[0].width = arg2[0].height = 0;
+	arg2[1].x = arg2[1].y = 192;
 
 	if (modeset && (skip_test = skip_on_unsupported_nonblocking_modeset(display)))
 		goto cleanup;
@@ -885,8 +1015,28 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
 
+		if (!modeset)
+			flip_nonblocking(display, pipe, false, &fb_info);
+		else {
+			/*
+			 * There are 2 design issues that prevent us from doing
+			 * the test we would like here:
+			 *
+			 * - drm_event_vblank doesn't set crtc_id, so if we
+			 *   use events we don't know which pipe fired the event,
+			 *   no way to distinguish.
+			 * - Doing a modeset may add unrelated pipes, and fail with
+			 *   -EBUSY if a page flip is queued on one of those.
+			 *
+			 * Work around it by not setting an event, but doing a synchronous
+			 * commit to wait for completion, and queue the page flip and modeset
+			 * in the same commit.
+			 */
 
-		flip_nonblocking(display, pipe, false, &fb_info);
+			igt_plane_set_fb(igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY), &fb_info);
+			igt_output_set_pipe(output2, (nloops & 1) ? PIPE_NONE : pipe2);
+			igt_display_commit_atomic(display, DRM_MODE_ATOMIC_ALLOW_MODESET | DRM_MODE_ATOMIC_NONBLOCK, NULL);
+		}
 
 		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
@@ -895,24 +1045,20 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg2[nloops & 1]);
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg2[nloops & 1]);
+
+			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+			igt_set_timeout(1, "Stuck page flip");
+			igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+			igt_reset_timeout();
 		} else {
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
-
-			igt_output_set_pipe(output2, (nloops & 1) ? PIPE_NONE : pipe2);
-			igt_display_commit_atomic(display, DRM_MODE_ATOMIC_ALLOW_MODESET | DRM_MODE_ATOMIC_NONBLOCK, NULL);
-
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg2[nloops & 1]);
 		}
 
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-
-		igt_set_timeout(1, "Stuck page flip");
-		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
-		igt_reset_timeout();
-
 		if (modeset) {
-			/* wait for pending modeset to complete, to prevent -EBUSY */
+			/* wait for pending modeset and page flip to complete, to prevent -EBUSY */
+			display->pipes[pipe].mode_changed = true;
 			display->pipes[pipe2].mode_changed = true;
 			igt_display_commit2(display, COMMIT_ATOMIC);
 		}
@@ -957,20 +1103,7 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	target = 4096;
-	do {
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-		for (int n = 0; n < target; n++)
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		target /= 2;
-		if (get_vblank(display->drm_fd, pipe, 0) == vblank_start)
-			break;
-	} while (target);
-	igt_require(target > 1);
-
-	igt_debug("Using a target of %ld cursor updates per half-vblank (%u)\n",
-		  target, vrefresh);
+	target = get_cursor_updates_per_vblank(display, pipe, &arg[0]);
 
 	for (int i = 0; i < nloops; i++) {
 		shared[0] = 0;
@@ -996,28 +1129,28 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 
 		igt_assert_eq(read(display->drm_fd, &vbl, sizeof(vbl)), sizeof(vbl));
 		vblank_start = vblank_last = vbl.sequence;
-		for (int n = 0; n < vrefresh; n++) {
+		for (int n = 0; n < vrefresh / 2; n++) {
 			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
 
 			igt_assert_eq(read(display->drm_fd, &vbl, sizeof(vbl)), sizeof(vbl));
 			if (vbl.sequence != vblank_last + 1) {
-				igt_warn("page flip %d was delayed, missed %d frames\n",
+				igt_info("page flip %d was delayed, missed %d frames\n",
 					 n, vbl.sequence - vblank_last - 1);
 			}
 			vblank_last = vbl.sequence;
 		}
 
-		if (mode != flip_test_atomic_transitions &&
-		    mode != flip_test_atomic_transitions_varying_size)
-			igt_assert_eq(vbl.sequence, vblank_start + vrefresh);
+		if (!cursor_slowpath(mode))
+			igt_assert_lte(vbl.sequence, vblank_start + 5 * vrefresh / 8);
 
 		shared[0] = 1;
 		igt_waitchildren();
-		igt_assert_f(shared[0] > vrefresh*target,
-			     "completed %lu cursor updated in a period of 60 flips, "
-			     "we expect to complete approximately %lu updateds, "
+		igt_assert_f(shared[0] > vrefresh*target / 2,
+			     "completed %lu cursor updated in a period of %u flips, "
+			     "we expect to complete approximately %lu updates, "
 			     "with the threshold set at %lu\n",
-			     shared[0], 2ul*vrefresh*target, vrefresh*target);
+			     shared[0], vrefresh / 2,
+			     vrefresh*target, vrefresh*target / 2);
 	}
 
 	do_cleanup_display(display);
@@ -1030,123 +1163,108 @@ static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloo
 		igt_remove_fb(display->drm_fd, &cursor_fb2);
 }
 
-static void two_screens_cursor_vs_flip(igt_display_t *display, int nloops, bool modeset)
+static void two_screens_cursor_vs_flip(igt_display_t *display, int nloops, bool atomic)
 {
-	struct drm_mode_cursor arg[2], arg2[2];
+	struct drm_mode_cursor arg[2][2];
 	struct drm_event_vblank vbl;
-	struct igt_fb fb_info, fb2_info, cursor_fb;
-	unsigned vblank_start, vblank_last;
+	struct igt_fb fb_info[2], cursor_fb;
 	volatile unsigned long *shared;
-	int target;
-	enum pipe pipe = find_connected_pipe(display, false);
-	enum pipe pipe2 = find_connected_pipe(display, true);
-	igt_output_t *output2;
-	bool skip_test = false;
+	int target[2];
+	enum pipe pipe[2] = {
+		find_connected_pipe(display, false),
+		find_connected_pipe(display, true)
+	};
+	igt_output_t *outputs[2];
 
 	shared = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(shared != MAP_FAILED);
 
-	if (modeset)
+	if (atomic)
 		igt_require(display->is_atomic);
 
-	igt_require(set_fb_on_crtc(display, pipe, &fb_info));
-	igt_require((output2 = set_fb_on_crtc(display, pipe2, &fb2_info)));
+	igt_require((outputs[0] = set_fb_on_crtc(display, pipe[0], &fb_info[0])));
+	igt_require((outputs[1] = set_fb_on_crtc(display, pipe[1], &fb_info[1])));
 
 	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
-	set_cursor_on_pipe(display, pipe, &cursor_fb);
-	populate_cursor_args(display, pipe, arg, &cursor_fb);
 
-	arg[0].flags = arg[1].flags = DRM_MODE_CURSOR_BO;
-	arg[1].handle = 0;
-	arg[1].width = arg[1].height = 0;
+	set_cursor_on_pipe(display, pipe[0], &cursor_fb);
+	populate_cursor_args(display, pipe[0], arg[0], &cursor_fb);
+	arg[0][1].x = arg[0][1].y = 192;
 
-	set_cursor_on_pipe(display, pipe2, &cursor_fb);
-	populate_cursor_args(display, pipe2, arg2, &cursor_fb);
-
-	arg2[0].flags = arg2[1].flags = DRM_MODE_CURSOR_BO;
-	arg2[0].handle = 0;
-	arg2[0].width = arg2[0].height = 0;
-
-	if (modeset && (skip_test = skip_on_unsupported_nonblocking_modeset(display)))
-		goto cleanup;
+	set_cursor_on_pipe(display, pipe[1], &cursor_fb);
+	populate_cursor_args(display, pipe[1], arg[1], &cursor_fb);
+	arg[1][1].x =  arg[1][1].y = 192;
 
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	target = 4096;
-	do {
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
-
-		if (!modeset)
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg2[0]);
-
-		for (int n = 0; n < target; n++) {
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-		}
-		target /= 2;
-		if (get_vblank(display->drm_fd, pipe, 0) == vblank_start)
-			break;
-	} while (target);
-	igt_require(target > 1);
-
-	igt_debug("Using a target of %d cursor updates per half-vblank\n",
-		  target);
+	target[0] = get_cursor_updates_per_vblank(display, pipe[0], &arg[0][0]);
+	target[1] = get_cursor_updates_per_vblank(display, pipe[1], &arg[1][0]);
 
 	for (int i = 0; i < nloops; i++) {
+		unsigned long vrefresh[2];
+		unsigned vblank_start[2], vblank_last[2];
+		int done[2] = {};
+
+		vrefresh[0] = igt_output_get_mode(outputs[0])->vrefresh;
+		vrefresh[1] = igt_output_get_mode(outputs[1])->vrefresh;
+
 		shared[0] = 0;
-		igt_fork(child, 1) {
+		shared[1] = 0;
+		igt_fork(child, 2) {
 			unsigned long count = 0;
 
-			if (!modeset)
-				do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg2[i & 1]);
-			else {
-				igt_output_set_pipe(output2, (i & 1) ? pipe2 : PIPE_NONE);
-				igt_display_commit_atomic(display, DRM_MODE_ATOMIC_ALLOW_MODESET |
-							  DRM_MODE_ATOMIC_NONBLOCK, NULL);
-			}
-
-			while (!shared[0]) {
-				do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[i & 1]);
+			while (!shared[child]) {
+				do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[child][(i >> child) & 1]);
 				count++;
 			}
-			igt_debug("child: %lu cursor updates\n", count);
-			shared[0] = count;
+			igt_debug("child %i: %lu cursor updates\n", child, count);
+			shared[child] = count;
 		}
 
-		flip_nonblocking(display, pipe, modeset, &fb_info);
+		flip_nonblocking(display, pipe[0], atomic, &fb_info[0]);
+		flip_nonblocking(display, pipe[1], atomic, &fb_info[1]);
 
-		igt_assert_eq(read(display->drm_fd, &vbl, sizeof(vbl)), sizeof(vbl));
-		vblank_start = vblank_last = vbl.sequence;
-		for (int n = 0; n < 60; n++) {
-			flip_nonblocking(display, pipe, modeset, &fb_info);
+		for (int n = 0; n < vrefresh[0] / 2 + vrefresh[1] / 2; n++) {
+			int child;
 
 			igt_assert_eq(read(display->drm_fd, &vbl, sizeof(vbl)), sizeof(vbl));
-			if (vbl.sequence != vblank_last + 1) {
-				igt_warn("page flip %d was delayed, missed %d frames\n",
-					 n, vbl.sequence - vblank_last - 1);
+			child = vbl.user_data == (unsigned long)&fb_info[1];
+
+			if (!done[child]++)
+				vblank_start[child] = vbl.sequence;
+			else if (vbl.sequence != vblank_last[child] + 1)
+				igt_info("page flip %d was delayed, missed %d frames\n",
+					 done[child], vbl.sequence - vblank_last[child] - 1);
+
+			vblank_last[child] = vbl.sequence;
+
+			if (done[child] < vrefresh[child] / 2) {
+				flip_nonblocking(display, pipe[child], atomic, &fb_info[child]);
+			} else {
+				igt_assert_lte(vbl.sequence, vblank_start[child] + 5 * vrefresh[child] / 8);
+
+				shared[child] = 1;
 			}
-			vblank_last = vbl.sequence;
 		}
-		igt_assert_eq(vbl.sequence, vblank_start + 60);
+
+		igt_assert_eq(done[0], vrefresh[0] / 2);
+		igt_assert_eq(done[1], vrefresh[1] / 2);
 
-		shared[0] = 1;
 		igt_waitchildren();
-		igt_assert_f(shared[0] > 60*target,
-			     "completed %lu cursor updated in a period of 60 flips, "
-			     "we expect to complete approximately %lu updateds, "
-			     "with the threshold set at %lu\n",
-			     shared[0], 2*60ul*target, 60ul*target);
+		for (int child = 0; child < 2; child++)
+			igt_assert_f(shared[child] > vrefresh[child]*target[child] / 2,
+				    "completed %lu cursor updated in a period of %lu flips, "
+				    "we expect to complete approximately %lu updates, "
+				    "with the threshold set at %lu\n",
+				    shared[child], vrefresh[child] / 2,
+				    vrefresh[child]*target[child], vrefresh[child]*target[child] / 2);
 	}
 
-cleanup:
 	do_cleanup_display(display);
-	igt_remove_fb(display->drm_fd, &fb_info);
-	igt_remove_fb(display->drm_fd, &fb2_info);
+	igt_remove_fb(display->drm_fd, &fb_info[0]);
+	igt_remove_fb(display->drm_fd, &fb_info[1]);
 	igt_remove_fb(display->drm_fd, &cursor_fb);
 	munmap((void *)shared, 4096);
-
-	if (skip_test)
-		igt_skip("Nonblocking modeset is not supported by this kernel\n");
 }
 
 static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
@@ -1167,23 +1285,18 @@ static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
 	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
 	populate_cursor_args(display, pipe, arg, &cursor_fb);
 
-	arg[0].flags = arg[1].flags |= DRM_MODE_CURSOR_BO;
-	arg[1].handle = 0;
-	arg[1].width = arg[1].height = 0;
-
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-
-	/* Collect reference crc with cursor disabled. */
-	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
+	pipe_crc = igt_pipe_crc_new(display->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
-	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+	/* Collect reference crcs, crcs[0] last. */
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[1]);
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
 
-	/* Collect reference crc with cursor enabled. */
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 	igt_pipe_crc_collect_crc(pipe_crc, &crcs[0]);
 
 	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
@@ -1220,6 +1333,8 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	unsigned vblank_start;
 	enum pipe pipe = find_connected_pipe(display, false);
 	igt_pipe_crc_t *pipe_crc;
+	igt_pipe_t *pipe_connected = &display->pipes[pipe];
+	igt_plane_t *plane_primary = igt_pipe_get_plane_type(pipe_connected, DRM_PLANE_TYPE_PRIMARY);
 	igt_crc_t crcs[3];
 
 	if (atomic)
@@ -1232,23 +1347,18 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
 	populate_cursor_args(display, pipe, arg, &cursor_fb);
 
-	arg[0].flags = arg[1].flags |= DRM_MODE_CURSOR_BO;
-	arg[1].handle = 0;
-	arg[1].width = arg[1].height = 0;
-
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-
-	/* Collect reference crc with cursor disabled. */
-	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
+	pipe_crc = igt_pipe_crc_new(display->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	set_cursor_on_pipe(display, pipe, &cursor_fb);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
-	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+	/* Collect reference crcs, crc[0] last for the loop. */
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[1]);
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
 
-	/* Collect reference crc with cursor enabled. */
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 	igt_pipe_crc_collect_crc(pipe_crc, &crcs[0]);
 
 	/*
@@ -1256,10 +1366,10 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 	  * setting the correct cache level, else we get a stall in the
 	  * page flip handler.
 	  */
-	igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[1]);
+	igt_plane_set_fb(plane_primary, &fb_info[1]);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
-	igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[0]);
+	igt_plane_set_fb(plane_primary, &fb_info[0]);
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
 	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
@@ -1285,7 +1395,7 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 
 		igt_assert_lte(vblank_start + 1, get_vblank(display->drm_fd, pipe, 0));
 
-		igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[0]);
+		igt_plane_set_fb(plane_primary, &fb_info[0]);
 		igt_display_commit2(display, COMMIT_UNIVERSAL);
 
 		igt_assert_crc_equal(&crcs[i], &crcs[2]);
@@ -1361,28 +1471,34 @@ igt_main
 		nonblocking_modeset_vs_cursor(&display, 16);
 
 	igt_subtest("2x-flip-vs-cursor-legacy")
-		two_screens_flip_vs_cursor(&display, 8, false);
+		two_screens_flip_vs_cursor(&display, 8, false, false);
+
+	igt_subtest("2x-flip-vs-cursor-atomic")
+		two_screens_flip_vs_cursor(&display, 4, false, true);
 
 	igt_subtest("2x-cursor-vs-flip-legacy")
-		two_screens_cursor_vs_flip(&display, 4, false);
+		two_screens_cursor_vs_flip(&display, 8, false);
 
 	igt_subtest("2x-long-flip-vs-cursor-legacy")
-		two_screens_flip_vs_cursor(&display, 150, false);
+		two_screens_flip_vs_cursor(&display, 150, false, false);
+
+	igt_subtest("2x-long-flip-vs-cursor-atomic")
+		two_screens_flip_vs_cursor(&display, 150, false, true);
 
 	igt_subtest("2x-long-cursor-vs-flip-legacy")
-		two_screens_cursor_vs_flip(&display, 150, false);
+		two_screens_cursor_vs_flip(&display, 50, false);
 
 	igt_subtest("2x-nonblocking-modeset-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 8, true);
+		two_screens_flip_vs_cursor(&display, 8, true, true);
 
-	igt_subtest("2x-cursor-vs-nonblocking-modeset-atomic")
-		two_screens_cursor_vs_flip(&display, 4, true);
+	igt_subtest("2x-cursor-vs-flip-atomic")
+		two_screens_cursor_vs_flip(&display, 8, true);
 
 	igt_subtest("2x-long-nonblocking-modeset-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 150, true);
+		two_screens_flip_vs_cursor(&display, 150, true, true);
 
-	igt_subtest("2x-long-cursor-vs-nonblocking-modeset-atomic")
-		two_screens_cursor_vs_flip(&display, 150, true);
+	igt_subtest("2x-long-cursor-vs-flip-atomic")
+		two_screens_cursor_vs_flip(&display, 50, true);
 
 	igt_subtest("flip-vs-cursor-crc-legacy")
 		flip_vs_cursor_crc(&display, false);
@@ -1419,9 +1535,13 @@ igt_main
 		igt_subtest_f("%sflip-before-cursor-%s", prefix, modes[i])
 			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR, 0);
 
-		igt_subtest_f("%sbusy-flip-before-cursor-%s", prefix, modes[i])
-			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR,
-					  BASIC_BUSY);
+		if (!cursor_slowpath(i)) {
+			igt_subtest_f("%sbusy-flip-before-cursor-%s", prefix, modes[i]) {
+				igt_require_gem(display.drm_fd);
+				basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR,
+						  BASIC_BUSY);
+			}
+		}
 
 		igt_subtest_f("%sflip-after-cursor-%s", prefix, modes[i])
 			basic_flip_cursor(&display, i, FLIP_AFTER_CURSOR, 0);
@@ -1429,7 +1549,7 @@ igt_main
 		igt_subtest_f("flip-vs-cursor-%s", modes[i])
 			flip_vs_cursor(&display, i, 150);
 		igt_subtest_f("cursor-vs-flip-%s", modes[i])
-			cursor_vs_flip(&display, i, 150);
+			cursor_vs_flip(&display, i, 50);
 
 		igt_subtest_f("cursorA-vs-flipA-%s", modes[i])
 			flip(&display, 0, 0, 10, i);
