@@ -23,14 +23,17 @@
  */
 
 #include "igt.h"
+#include "igt_sysfs.h"
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+#include <fcntl.h>
 
 
 typedef struct {
 	int drm_fd;
+	int debugfs;
 	igt_display_t display;
 	struct igt_fb fb;
 } data_t;
@@ -45,17 +48,15 @@ static struct {
 
 static void test_bad_command(data_t *data, const char *cmd)
 {
-	FILE *ctl;
-	size_t written;
+	igt_assert(!igt_sysfs_set(data->debugfs, "i915_display_crc_ctl", cmd));
+	igt_skip_on(errno == ENOENT);
+	igt_assert(errno == EINVAL);
+}
 
-	ctl = igt_debugfs_fopen("i915_display_crc_ctl", "r+");
-	written = fwrite(cmd, 1, strlen(cmd), ctl);
-	fflush(ctl);
-	igt_assert_eq(written, strlen(cmd));
-	igt_assert(ferror(ctl));
-	igt_assert_eq(errno, EINVAL);
-
-	fclose(ctl);
+static void test_bad_source(data_t *data)
+{
+	igt_assert(igt_sysfs_set(data->debugfs, "crtc-0/crc/control", "foo"));
+	igt_assert(openat(data->debugfs, "crtc-0/crc/data", O_WRONLY) == -1);
 }
 
 #define N_CRCS	3
@@ -63,14 +64,13 @@ static void test_bad_command(data_t *data, const char *cmd)
 #define TEST_SEQUENCE (1<<0)
 #define TEST_NONBLOCK (1<<1)
 
-static int
+static void
 test_read_crc_for_output(data_t *data, int pipe, igt_output_t *output,
 			 unsigned flags)
 {
 	igt_display_t *display = &data->display;
 	igt_plane_t *primary;
 	drmModeModeInfo *mode;
-	igt_pipe_crc_t *pipe_crc;
 	igt_crc_t *crcs = NULL;
 	int c, j;
 
@@ -79,12 +79,6 @@ test_read_crc_for_output(data_t *data, int pipe, igt_output_t *output,
 		int n_crcs;
 
 		igt_output_set_pipe(output, pipe);
-		igt_display_commit(display);
-
-		if (!output->valid) {
-			igt_output_set_pipe(output, PIPE_ANY);
-			return 0;
-		}
 
 		igt_debug("Clearing the fb with color (%.02lf,%.02lf,%.02lf)\n",
 			  colors[c].r, colors[c].g, colors[c].b);
@@ -104,30 +98,35 @@ test_read_crc_for_output(data_t *data, int pipe, igt_output_t *output,
 
 		igt_display_commit(display);
 
-		if (flags & TEST_NONBLOCK)
-			pipe_crc = igt_pipe_crc_new_nonblock(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-		else
-			pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-
-		igt_pipe_crc_start(pipe_crc);
-
 		/* wait for N_CRCS vblanks and the corresponding N_CRCS CRCs */
 		if (flags & TEST_NONBLOCK) {
-			int i;
+			igt_pipe_crc_t *pipe_crc;
 
-			for (i = 0; i < N_CRCS; i++)
-				igt_wait_for_vblank(data->drm_fd, pipe);
+			pipe_crc = igt_pipe_crc_new_nonblock(data->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+			igt_wait_for_vblank(data->drm_fd, pipe);
+			igt_pipe_crc_start(pipe_crc);
+
+			igt_wait_for_vblank_count(data->drm_fd, pipe, N_CRCS);
+			n_crcs = igt_pipe_crc_get_crcs(pipe_crc, N_CRCS+1, &crcs);
+			igt_pipe_crc_stop(pipe_crc);
+			igt_pipe_crc_free(pipe_crc);
 
-			n_crcs = igt_pipe_crc_get_crcs(pipe_crc, N_CRCS * 3, &crcs);
 			/* allow a one frame difference */
-			igt_assert_lte(n_crcs, N_CRCS + 1);
-			igt_assert_lte(N_CRCS, n_crcs + 1);
+			igt_assert_lte(N_CRCS, n_crcs);
 		} else {
+			igt_pipe_crc_t *pipe_crc;
+
+			pipe_crc = igt_pipe_crc_new(data->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+			igt_pipe_crc_start(pipe_crc);
+
 			n_crcs = igt_pipe_crc_get_crcs(pipe_crc, N_CRCS, &crcs);
+
+			igt_pipe_crc_stop(pipe_crc);
+			igt_pipe_crc_free(pipe_crc);
+
 			igt_assert_eq(n_crcs, N_CRCS);
 		}
 
-		igt_pipe_crc_stop(pipe_crc);
 
 		/*
 		 * save the CRC in colors so it can be compared to the CRC of
@@ -148,14 +147,11 @@ test_read_crc_for_output(data_t *data, int pipe, igt_output_t *output,
 				igt_assert_eq(crcs[j].frame + 1, crcs[j + 1].frame);
 
 		free(crcs);
-		igt_pipe_crc_free(pipe_crc);
 		igt_remove_fb(data->drm_fd, &data->fb);
 		igt_plane_set_fb(primary, NULL);
 
 		igt_output_set_pipe(output, PIPE_ANY);
 	}
-
-	return 1;
 }
 
 static void test_read_crc(data_t *data, int pipe, unsigned flags)
@@ -166,13 +162,14 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 
 	igt_skip_on(pipe >= data->display.n_pipes);
 
-	for_each_connected_output(display, output) {
+	for_each_valid_output_on_pipe(display, pipe, output) {
 
 		igt_info("%s: Testing connector %s using pipe %s\n",
 			 igt_subtest_name(), igt_output_name(output),
 			 kmstest_pipe_name(pipe));
 
-		valid_connectors += test_read_crc_for_output(data, pipe, output, flags);
+		test_read_crc_for_output(data, pipe, output, flags);
+		valid_connectors ++;
 	}
 
 	igt_require_f(valid_connectors, "No connector found for pipe %i\n", pipe);
@@ -185,22 +182,23 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
+		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		igt_enable_connectors();
 
 		kmstest_set_vt_graphics_mode();
 
-		igt_require_pipe_crc();
+		igt_require_pipe_crc(data.drm_fd);
 
 		igt_display_init(&data.display, data.drm_fd);
+		data.debugfs = igt_debugfs_dir(data.drm_fd);
 	}
 
 	igt_subtest("bad-pipe")
 		test_bad_command(&data, "pipe D none");
 
 	igt_subtest("bad-source")
-		test_bad_command(&data, "pipe A foo");
+		test_bad_source(&data);
 
 	igt_subtest("bad-nb-words-1")
 		test_bad_command(&data, "pipe foo");
@@ -230,11 +228,15 @@ igt_main
 		}
 
 		igt_subtest_f("hang-read-crc-pipe-%c", 'A'+i) {
-			igt_hang_t hang =
-				igt_hang_ring(data.drm_fd, I915_EXEC_RENDER);
+			igt_hang_t hang = igt_allow_hang(data.drm_fd, 0, 0);
+
 			test_read_crc(&data, i, 0);
-			igt_post_hang_ring(data.drm_fd, hang);
+
+			igt_force_gpu_reset(data.drm_fd);
+
 			test_read_crc(&data, i, 0);
+
+			igt_disallow_hang(data.drm_fd, hang);
 		}
 	}
 
