@@ -96,7 +96,6 @@ unsigned int plane_fb_id;
 unsigned int plane_crtc_id;
 unsigned int plane_id;
 int plane_width, plane_height;
-static const uint32_t SPRITE_COLOR_KEY = 0x00aaaaaa;
 
 /*
  * Mode setting with the kernel interfaces is a bit of a chore.
@@ -112,7 +111,6 @@ struct connector {
 	drmModeEncoder *encoder;
 	drmModeConnector *connector;
 	int crtc;
-	int crtc_idx;
 	int pipe;
 };
 
@@ -211,7 +209,6 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 	c->connector = config.connector;
 	c->encoder = config.encoder;
 	c->crtc = config.crtc->crtc_id;
-	c->crtc_idx = config.crtc_idx;
 	c->pipe = config.pipe;
 
 	if (mode_num != -1) {
@@ -497,7 +494,7 @@ int update_display(bool probe)
 
 			if (test_preferred_mode || force_mode ||
 			    specified_mode_num != -1)
-				crtc_idx_mask &= ~(1 << connector->crtc_idx);
+				crtc_idx_mask &= ~(1 << connector->pipe);
 
 		}
 	}
