@@ -51,6 +51,7 @@
 #include <err.h>
 #include <assert.h>
 #include <intel_bufmgr.h>
+#include <zlib.h>
 
 #include "intel_chipset.h"
 #include "intel_io.h"
@@ -433,82 +434,222 @@ print_fault_data(unsigned devid, uint32_t data1, uint32_t data0)
 }
 
 #define MAX_RINGS 10 /* I really hope this never... */
-uint32_t head[MAX_RINGS];
-int head_ndx = 0;
-int num_rings = 0;
-static void print_batch(int is_batch, const char *ring_name, uint32_t gtt_offset)
-{
-	const char *buffer_type[2] = {  "ringbuffer", "batchbuffer" };
-	if (is_batch || !num_rings)
-		printf("%s (%s) at 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset);
-	else
-		printf("%s (%s) at 0x%08x; HEAD points to: 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset, head[head_ndx++ % num_rings] + gtt_offset);
-}
 
-static void decode(struct drm_intel_decode *ctx, bool is_batch,
-		   const char *ring_name, uint32_t gtt_offset, uint32_t *data,
-		   int *count)
+static void decode(struct drm_intel_decode *ctx,
+		   const char *buffer_name,
+		   const char *ring_name,
+		   uint64_t gtt_offset,
+		   uint32_t head_offset,
+		   uint32_t *data, int *count,
+		   int decode)
 {
 	if (!*count)
 		return;
 
-	print_batch(is_batch, ring_name, gtt_offset);
-	drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset, *count);
-	drm_intel_decode(ctx);
+	printf("%s (%s) at 0x%08x_%08x", buffer_name, ring_name,
+	       (unsigned)(gtt_offset >> 32),
+	       (unsigned)(gtt_offset & 0xffffffff));
+	if (head_offset != -1)
+		printf("; HEAD points to: 0x%08x_%08x",
+		       (unsigned)((head_offset + gtt_offset) >> 32),
+		       (unsigned)((head_offset + gtt_offset) & 0xffffffff));
+	printf("\n");
+
+	if (decode) {
+		drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset,
+						   *count);
+		drm_intel_decode(ctx);
+	} else {
+		for (int i = 0; i + 4 <= *count; i += 4)
+			printf("[%04x] %08x %08x %08x %08x\n",
+			       4*i, data[i], data[i+1], data[i+2], data[i+3]);
+	}
 	*count = 0;
 }
 
+static int zlib_inflate(uint32_t **ptr, int len)
+{
+	struct z_stream_s zstream;
+	void *out;
+
+	memset(&zstream, 0, sizeof(zstream));
+
+	zstream.next_in = (unsigned char *)*ptr;
+	zstream.avail_in = 4*len;
+
+	if (inflateInit(&zstream) != Z_OK)
+		return 0;
+
+	out = malloc(128*4096); /* approximate obj size */
+	zstream.next_out = out;
+	zstream.avail_out = 128*4096;
+
+	do {
+		switch (inflate(&zstream, Z_SYNC_FLUSH)) {
+		case Z_STREAM_END:
+			goto end;
+		case Z_OK:
+			break;
+		default:
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		if (zstream.avail_out)
+			break;
+
+		out = realloc(out, 2*zstream.total_out);
+		if (out == NULL) {
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		zstream.next_out = (unsigned char *)out + zstream.total_out;
+		zstream.avail_out = zstream.total_out;
+	} while (1);
+end:
+	inflateEnd(&zstream);
+	free(*ptr);
+	*ptr = out;
+	return zstream.total_out / 4;
+}
+
+static int ascii85_decode(const char *in, uint32_t **out, bool inflate)
+{
+	int len = 0, size = 1024;
+
+	*out = realloc(*out, sizeof(uint32_t)*size);
+	if (*out == NULL)
+		return 0;
+
+	while (*in >= '!' && *in <= 'z') {
+		uint32_t v = 0;
+
+		if (len == size) {
+			size *= 2;
+			*out = realloc(*out, sizeof(uint32_t)*size);
+			if (*out == NULL)
+				return 0;
+		}
+
+		if (*in == 'z') {
+			in++;
+		} else {
+			v += in[0] - 33; v *= 85;
+			v += in[1] - 33; v *= 85;
+			v += in[2] - 33; v *= 85;
+			v += in[3] - 33; v *= 85;
+			v += in[4] - 33;
+			in += 5;
+		}
+		(*out)[len++] = v;
+	}
+
+	if (!inflate)
+		return len;
+
+	return zlib_inflate(out, len);
+}
+
 static void
 read_data_file(FILE *file)
 {
 	struct drm_intel_decode *decode_ctx = NULL;
 	uint32_t devid = PCI_CHIP_I855_GM;
 	uint32_t *data = NULL;
+	uint32_t head[MAX_RINGS];
+	int head_idx = 0;
+	int num_rings = 0;
 	long long unsigned fence;
-	int data_size = 0, count = 0, line_number = 0, matched;
+	int data_size = 0, count = 0, matched;
 	char *line = NULL;
 	size_t line_size;
 	uint32_t offset, value, ring_length = 0;
-	uint32_t gtt_offset = 0, new_gtt_offset;
+	uint64_t gtt_offset = 0;
+	uint32_t head_offset = -1;
+	const char *buffer_name = "batch buffer";
 	char *ring_name = NULL;
-	int is_batch = 1;
+	int do_decode = 1;
 
 	while (getline(&line, &line_size, file) > 0) {
 		char *dashes;
-		line_number++;
+
+		if (line[0] == ':' || line[0] == '~') {
+			count = ascii85_decode(line+1, &data, line[0] == ':');
+			if (count == 0)
+				fprintf(stderr, "ASCII85 decode failed (%s - %s).\n",
+					ring_name, buffer_name);
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
+			continue;
+		}
 
 		dashes = strstr(line, "---");
 		if (dashes) {
-			char *new_ring_name = malloc(dashes - line);
+			const struct {
+				const char *match;
+				const char *name;
+				int do_decode;
+			} buffers[] = {
+				{ "ringbuffer", "ring", 1 },
+				{ "gtt_offset", "batch", 1 },
+				{ "hw context", "HW context", 1 },
+				{ "hw status", "HW status", 0 },
+				{ "wa context", "WA context", 1 },
+				{ "wa batchbuffer", "WA batch", 1 },
+				{ "user", "user", 0 },
+				{ "semaphores", "semaphores", 0 },
+				{ "guc log buffer", "GuC log", 0 },
+				{ },
+			}, *b;
+			char *new_ring_name;
+
+			new_ring_name = malloc(dashes - line);
 			strncpy(new_ring_name, line, dashes - line);
 			new_ring_name[dashes - line - 1] = '\0';
 
-			if (num_rings == -1)
-				num_rings = head_ndx;
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
+			gtt_offset = 0;
+			head_offset = -1;
 
-			matched = sscanf(dashes, "--- gtt_offset = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				decode(decode_ctx, is_batch, ring_name,
-				       gtt_offset, data, &count);
-				gtt_offset = new_gtt_offset;
-				is_batch = 1;
-				free(ring_name);
-				ring_name = new_ring_name;
-				continue;
+			free(ring_name);
+			ring_name = new_ring_name;
+
+			dashes += 4;
+			for (b = buffers; b->match; b++) {
+				uint32_t lo, hi;
+
+				if (strncasecmp(dashes, b->match,
+						strlen(b->match)))
+					continue;
+
+				dashes = strchr(dashes, '=');
+				if (!dashes)
+					break;
+
+				matched = sscanf(dashes, "= 0x%08x %08x\n",
+						 &hi, &lo);
+				if (matched > 0) {
+					gtt_offset = hi;
+					if (matched == 2) {
+						gtt_offset <<= 32;
+						gtt_offset |= lo;
+					}
+				}
+
+				do_decode = b->do_decode;
+				buffer_name = b->name;
+				if (b == buffers)
+					head_offset = head[head_idx++];
+				break;
 			}
 
-			matched = sscanf(dashes, "--- ringbuffer = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				decode(decode_ctx, is_batch, ring_name,
-				       gtt_offset, data, &count);
-				gtt_offset = new_gtt_offset;
-				is_batch = 0;
-				free(ring_name);
-				ring_name = new_ring_name;
-				continue;
-			}
+			continue;
 		}
 
 		matched = sscanf(line, "%08x : %08x", &offset, &value);
@@ -516,8 +657,10 @@ read_data_file(FILE *file)
 			unsigned int reg, reg2;
 
 			/* display reg section is after the ringbuffers, don't mix them */
-			decode(decode_ctx, is_batch, ring_name, gtt_offset,
-			       data, &count);
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
 
 			printf("%s", line);
 
@@ -597,13 +740,40 @@ read_data_file(FILE *file)
 		data[count-1] = value;
 	}
 
-	decode(decode_ctx, is_batch, ring_name, gtt_offset, data, &count);
+	decode(decode_ctx,
+	       buffer_name, ring_name,
+	       gtt_offset, head_offset,
+	       data, &count, do_decode);
 
 	free(data);
 	free(line);
 	free(ring_name);
 }
 
+static void setup_pager(void)
+{
+	int fds[2];
+
+	if (pipe(fds) == -1)
+		return;
+
+	switch (fork()) {
+	case -1:
+		break;
+	case 0:
+		close(fds[1]);
+		dup2(fds[0], 0);
+		execlp("less", "less", "-FRSi", NULL);
+		break;
+
+	default:
+		close(fds[0]);
+		dup2(fds[1], 1);
+		close(fds[1]);
+		break;
+	}
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -629,6 +799,9 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (isatty(1))
+		setup_pager();
+
 	if (argc == 1) {
 		if (isatty(0)) {
 			path = "/sys/class/drm/card0/error";
@@ -702,4 +875,4 @@ main(int argc, char *argv[])
 	return 0;
 }
 
-/* vim: set ts=8 sw=8 tw=0 noet :*/
+/* vim: set ts=8 sw=8 tw=0 cino=:0,(0 noet :*/
