@@ -27,7 +27,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -40,8 +42,11 @@
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #endif
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "instdone.h"
+#include "intel_reg.h"
+#include "intel_chipset.h"
+#include "drmtest.h"
 
 #define  FORCEWAKE	    0xA18C
 #define  FORCEWAKE_ACK	    0x130090
@@ -142,7 +147,7 @@ update_idle_bit(struct top_bit *top_bit)
 {
 	uint32_t reg_val;
 
-	if (top_bit->bit->reg == INST_DONE_1)
+	if (top_bit->bit->reg == INSTDONE_1)
 		reg_val = instdone1;
 	else
 		reg_val = instdone;
@@ -406,6 +411,7 @@ usage(const char *appname)
 int main(int argc, char **argv)
 {
 	uint32_t devid;
+	int drm_fd;
 	struct pci_device *pci_dev;
 	struct ring render_ring = {
 		.name = "render",
@@ -466,12 +472,10 @@ int main(int argc, char **argv)
 			break;
 		}
 	}
-	argc -= optind;
-	argv += optind;
 
 	pci_dev = intel_get_pci_device();
 	devid = pci_dev->device_id;
-	intel_get_mmio(pci_dev);
+	intel_mmio_use_pci_bar(pci_dev);
 	init_instdone_definitions(devid);
 
 	/* Do we have a command to run? */
@@ -508,13 +512,16 @@ int main(int argc, char **argv)
 		top_bits_sorted[i] = &top_bits[i];
 	}
 
+	/* Just to make sure we open the right debugfs files */
+	drm_fd = drm_open_driver_master(DRIVER_INTEL);
+
 	/* Grab access to the registers */
-	intel_register_access_init(pci_dev, 1);
+	intel_register_access_init(pci_dev, 0, drm_fd);
 
 	ring_init(&render_ring);
 	if (IS_GEN4(devid) || IS_GEN5(devid))
 		ring_init(&bsd_ring);
-	if (IS_GEN6(devid)) {
+	if (IS_GEN6(devid) || IS_GEN7(devid)) {
 		ring_init(&bsd6_ring);
 		ring_init(&blt_ring);
 	}
@@ -559,10 +566,10 @@ int main(int argc, char **argv)
 			long long interval;
 			ti = gettime();
 			if (IS_965(devid)) {
-				instdone = INREG(INST_DONE_I965);
-				instdone1 = INREG(INST_DONE_1);
+				instdone = INREG(INSTDONE_I965);
+				instdone1 = INREG(INSTDONE_1);
 			} else
-				instdone = INREG(INST_DONE);
+				instdone = INREG(INSTDONE);
 
 			for (j = 0; j < num_instdone_bits; j++)
 				update_idle_bit(&top_bits[j]);
@@ -677,7 +684,7 @@ int main(int argc, char **argv)
 
 			for (i = 0; i < MAX_NUM_TOP_BITS; i++) {
 				if (i < STATS_COUNT && HAS_STATS_REGS(devid)) {
-					fprintf(output, "%lu\t",
+					fprintf(output, "%"PRIu64"\t",
 						   stats[i] - last_stats[i]);
 					last_stats[i] = stats[i];
 				}
@@ -712,5 +719,6 @@ int main(int argc, char **argv)
 	fclose(output);
 
 	intel_register_access_fini();
+	close(drm_fd);
 	return 0;
 }
